@@ -2,9 +2,9 @@
 
 """Main entry point for MiniVCP.
 
-This module contains the code necessary to be able to launch MiniVCP
-directly from the command line, without using qtpyvcp. It handles
-parsing command line args and starting the main application.
+This module contains the code required to run MiniVCP directly, without
+having to pass it as an argument to qtpyvcp. It handles parsing command
+line args and starting the main application.
 
 Example:
     Assuming the dir this file is located in is on the PATH, you can
@@ -33,7 +33,8 @@ def main(opts=None):
                           vcp_name='Mini',
                           vcp_version=__version__)
 
-    qtpyvcp.run_vcp(opts, VCP_CONFIG_FILE)
+    # run the main application
+    qtpyvcp.app.run(opts, VCP_CONFIG_FILE)
 
 
 if __name__ == '__main__':
