@@ -1,11 +1,8 @@
-#!/usr/bin/env python
-# coding: utf-8
-
-import sys
-import time
 import linuxcnc
 from qtpy.QtWidgets import QComboBox
 
+from qtpyvcp.utilities.settings import setting
+
 # Set up logging
 from qtpyvcp.utilities import logger
 LOG = logger.getLogger(__name__)
@@ -28,19 +25,35 @@ class estop:
     """E-Stop action group"""
     @staticmethod
     def activate():
-        """Set E-Stop active"""
+        """Set E-Stop active
+
+        ActionButton syntax::
+
+            machine.estop.activate
+        """
         LOG.debug("Setting state red<ESTOP>")
         CMD.state(linuxcnc.STATE_ESTOP)
 
     @staticmethod
     def reset():
-        """Resets E-Stop"""
+        """Resets E-Stop
+
+        ActionButton syntax::
+
+            machine.estop.reset
+        """
         LOG.debug("Setting state green<ESTOP_RESET>")
         CMD.state(linuxcnc.STATE_ESTOP_RESET)
 
     @staticmethod
     def toggle():
-        """Toggles E-Stop state"""
+        """Toggles E-Stop state
+
+        ActionButton syntax::
+
+            machine.estop.toggle
+        """
+
         if estop.is_activated():
             estop.reset()
         else:
@@ -74,19 +87,34 @@ class power:
     """Power action group"""
     @staticmethod
     def on():
-        """Turns machine power On"""
+        """Turns machine power On
+
+        ActionButton syntax::
+
+            machine.power.on
+        """
         LOG.debug("Setting state green<ON>")
         CMD.state(linuxcnc.STATE_ON)
 
     @staticmethod
     def off():
-        """Turns machine power Off"""
+        """Turns machine power Off
+
+        ActionButton syntax::
+
+            machine.power.off
+        """
         LOG.debug("Setting state red<OFF>")
         CMD.state(linuxcnc.STATE_OFF)
 
     @staticmethod
     def toggle():
-        """Toggles machine power On/Off"""
+        """Toggles machine power On/Off
+
+        ActionButton syntax::
+
+            machine.power.toggle
+        """
         if power.is_on():
             power.off()
         else:
@@ -122,7 +150,7 @@ def _power_bindOk(widget):
     _power_ok(widget)
     widget.setChecked(STAT.task_state == linuxcnc.STATE_ON)
     STATUS.estop.onValueChanged(lambda: _power_ok(widget))
-    STATUS.on.connect(lambda v: widget.setChecked(v))
+    STATUS.on.notify(lambda v: widget.setChecked(v))
 
 power.on.ok = power.off.ok = power.toggle.ok = _power_ok
 power.on.bindOk = power.off.bindOk = power.toggle.bindOk = _power_bindOk
@@ -146,15 +174,22 @@ def issue_mdi(command, reset=True):
     """Issue an MDI command.
 
         An MDI command can be issued any time the machine is homed (if not
-        NOT_FORCE_HOMING in the INI) and the interpreter is IDLE.  The task
+        NO_FORCE_HOMING in the INI) and the interpreter is IDLE.  The task
         mode will automatically be switched to MDI prior to issuing the command
         and will be returned to the previous mode when the interpreter becomes IDLE.
 
-    Args:
-        command (str) : A valid RS274 gcode command string.
-        rest (bool, optional): Whether to reset the Task Mode to the state
-            the machine was in prior to issuing the MDI command.
+        ActionButton syntax to issue G0 X5:
+        ::
 
+            machine.issue_mdi:G0X5
+
+        It is simpler to use the MDIButton, examples in the Widgets section.
+
+    Args:
+        command (str) : A valid RS274 gcode command string. Multiple MDI commands
+            can be separated with a ``;`` and will be issued sequentially.
+        reset (bool, optional): Whether to reset the Task Mode to the state
+            the machine was in prior to issuing the MDI command.
     """
     if reset:
         # save the previous mode
@@ -167,15 +202,17 @@ def issue_mdi(command, reset=True):
         STATUS.old['interp_state'] = -1
 
     if setTaskMode(linuxcnc.MODE_MDI):
-        LOG.info("Issuind MDI command: {}".format(command))
-        CMD.mdi(command)
+        # issue multiple MDI commands separated by ';'
+        for cmd in command.strip().split(';'):
+            LOG.info("Issuing MDI command: %s", cmd)
+            CMD.mdi(cmd)
     else:
         LOG.error("Failed to issue MDI command: {}".format(command))
 
 def _issue_mdi_ok(mdi_cmd='', widget=None):
     if STAT.task_state == linuxcnc.STATE_ON \
-        and STATUS.allHomed() \
-        and STAT.interp_state == linuxcnc.INTERP_IDLE:
+                          and STATUS.allHomed() \
+                          and STAT.interp_state == linuxcnc.INTERP_IDLE:
 
         ok = True
         msg = ""
@@ -194,6 +231,7 @@ def _issue_mdi_ok(mdi_cmd='', widget=None):
     return ok
 
 def _issue_mdi_bindOk(mdi_cmd='', widget=None):
+    _issue_mdi_ok(mdi_cmd=mdi_cmd, widget=widget)
     STATUS.task_state.onValueChanged(lambda: _issue_mdi_ok(mdi_cmd=mdi_cmd, widget=widget))
     STATUS.interp_state.onValueChanged(lambda: _issue_mdi_ok(mdi_cmd=mdi_cmd, widget=widget))
     STATUS.homed.onValueChanged(lambda: _issue_mdi_ok(mdi_cmd=mdi_cmd, widget=widget))
@@ -213,12 +251,12 @@ def _set_work_coord_bindOk(coord='', widget=None):
     _issue_mdi_bindOk(coord, widget=widget)
     if isinstance(widget, QComboBox):
         widget.setCurrentText(coord)
-        widget.setCurrentText(STATUS.g5x_index.to_str(STATUS.g5x_index.value))
-        STATUS.g5x_index.onValueChanged(lambda wc: widget.setCurrentText(STATUS.g5x_index.to_str(wc)))
+        widget.setCurrentText(STATUS.g5x_index.getString())
+        STATUS.g5x_index.notify(lambda g5x: widget.setCurrentText(g5x), 'string')
     else:
         widget.setCheckable(True)
-        widget.setChecked(STATUS.g5x_index.to_str(STATUS.g5x_index.value) == coord)
-        STATUS.g5x_index.onValueChanged(lambda wc: widget.setChecked(STATUS.g5x_index.to_str(wc) == coord))
+        widget.setChecked(STATUS.g5x_index.getString() == coord)
+        STATUS.g5x_index.notify(lambda g5x: widget.setChecked(g5x == coord), 'string')
 
 set_work_coord.ok = _issue_mdi_ok
 set_work_coord.bindOk = _set_work_coord_bindOk
@@ -227,62 +265,61 @@ set_work_coord.bindOk = _set_work_coord_bindOk
 # FEED HOLD action
 # -------------------------------------------------------------------------
 class feedhold:
+    """Feed Hold action Group"""
 
     # FIXME: Not sure what feedhold does, or how to turn it ON/OFF, if it even can be.
 
     @staticmethod
     def enable():
+        """Enables Feed Hold"""
         LOG.info("Setting feedhold ENABLED")
         CMD.set_feed_hold(1)
 
     @staticmethod
     def disable():
+        """Disables Feed Hold"""
         LOG.info("Setting feedhold DISABLED")
         CMD.set_feed_hold(0)
 
     @staticmethod
-    def toggle_enable():
+    def toggle():
+        """Toggles Feed Hold state"""
         if STAT.feed_hold_enabled:
             feedhold.disable()
         else:
             feedhold.enable()
 
-    @staticmethod
-    def on():
-        pass
-
-    @staticmethod
-    def off():
-        pass
-
-    @staticmethod
-    def toggle():
-        pass
-
 def _feed_hold_ok(widget=None):
-    return True
+    return STAT.task_state == linuxcnc.STATE_ON and STAT.interp_state == linuxcnc.INTERP_IDLE
 
 def _feed_hold_bindOk(widget):
-    pass
+    widget.setEnabled(STAT.task_state == linuxcnc.STATE_ON)
+    widget.setChecked(STAT.feed_hold_enabled)
+    STATUS.task_state.notify(lambda s: widget.setEnabled(s == linuxcnc.STATE_ON))
+    STATUS.feed_hold_enabled.notify(widget.setChecked)
 
-feedhold.enable.ok = feedhold.disable.ok = feedhold.toggle_enable.ok = _feed_hold_ok
-feedhold.enable.bindOk = feedhold.disable.bindOk = feedhold.toggle_enable.bindOk = _feed_hold_bindOk
+feedhold.enable.ok = feedhold.disable.ok = feedhold.toggle.ok = _feed_hold_ok
+feedhold.enable.bindOk = feedhold.disable.bindOk = feedhold.toggle.bindOk = _feed_hold_bindOk
 
 # -------------------------------------------------------------------------
 # FEED OVERRIDE actions
 # -------------------------------------------------------------------------
 
 class feed_override:
+    """Feed Override Group"""
     @staticmethod
     def enable():
+        """Feed Override Enable"""
         CMD.set_feed_override(True)
 
     @staticmethod
     def disable():
+        """Feed Override Disable"""
         CMD.set_feed_override(False)
 
     @staticmethod
     def toggle_enable():
+        """Feed Override Enable Toggle"""
         if STAT.feed_override_enabled:
             feed_override.disable()
         else:
@@ -290,10 +327,12 @@ class feed_override:
 
     @staticmethod
     def set(value):
+        """Feed Override Set Value"""
         CMD.feedrate(float(value) / 100)
 
     @staticmethod
     def reset():
+        """Feed Override Reset"""
         CMD.feedrate(1.0)
 
 def _feed_override_enable_ok(widget=None):
@@ -352,10 +391,19 @@ def _feed_override_bindOk(value=100, widget=None):
         # these will only work for QSlider or QSpinBox
         widget.setMinimum(0)
         widget.setMaximum(INFO.maxFeedOverride() * 100)
-        widget.setValue(100)
+
+        try:
+            widget.setSliderPosition(100)
+            STATUS.feedrate.onValueChanged(
+                lambda v: widget.setSliderPosition(v * 100))
+
+        except AttributeError:
+            widget.setValue(100)
+            STATUS.feedrate.onValueChanged(
+                lambda v: widget.setValue(v * 100))
+
         feed_override.set(100)
 
-        STATUS.feedrate.onValueChanged(lambda v: widget.setValue(v * 100))
     except AttributeError:
         pass
     except:
@@ -371,6 +419,7 @@ feed_override.enable.bindOk = feed_override.disable.bindOk = feed_override.toggl
 # -------------------------------------------------------------------------
 
 class rapid_override:
+    """Rapid Override Group"""
     @staticmethod
     def set(value):
         CMD.rapidrate(float(value) / 100)
@@ -405,10 +454,19 @@ def _rapid_override_bindOk(value=100, widget=None):
         # these will only work for QSlider or QSpinBox
         widget.setMinimum(0)
         widget.setMaximum(100)
-        widget.setValue(100)
+
+        try:
+            widget.setSliderPosition(100)
+            STATUS.rapidrate.onValueChanged(
+                lambda v: widget.setSliderPosition(v * 100))
+
+        except AttributeError:
+            STATUS.rapidrate.onValueChanged(
+                lambda v: widget.setValue(v * 100))
+            widget.setValue(100)
+
         rapid_override.set(100)
 
-        STATUS.rapidrate.onValueChanged(lambda v: widget.setValue(v * 100))
     except AttributeError:
         pass
     except:
@@ -422,12 +480,15 @@ rapid_override.set.bindOk = rapid_override.reset.bindOk = _rapid_override_bindOk
 # -------------------------------------------------------------------------
 
 class max_velocity:
+    """Max Velocity Group"""
     @staticmethod
     def set(value):
+        """Max Velocity Override Set Value"""
         CMD.maxvel(float(value) / 60)
 
     @staticmethod
     def reset():
+        """Max Velocity Override Reset Value"""
         CMD.maxvel(INFO.maxVelocity() / 60)
 
 def _max_velocity_ok(value=100, widget=None):
@@ -456,9 +517,17 @@ def _max_velocity_bindOk(value=100, widget=None):
         # these will only work for QSlider or QSpinBox
         widget.setMinimum(0)
         widget.setMaximum(INFO.maxVelocity())
-        widget.setValue(INFO.maxVelocity())
 
-        STATUS.max_velocity.onValueChanged(lambda v: widget.setValue(v * 60))
+        try:
+            widget.setSliderPosition(INFO.maxVelocity())
+            STATUS.max_velocity.onValueChanged(
+                lambda v: widget.setSliderPosition(v * 60))
+
+        except AttributeError:
+            widget.setValue(INFO.maxVelocity())
+            STATUS.max_velocity.onValueChanged(
+                lambda v: widget.setValue(v * 60))
+
     except AttributeError:
         pass
     except:
@@ -472,16 +541,41 @@ max_velocity.set.bindOk = max_velocity.reset.bindOk = _max_velocity_bindOk
 # set MODE actions
 # -------------------------------------------------------------------------
 class mode:
+    """Mode action group"""
     @staticmethod
     def manual():
+        """Change mode to Manual
+
+        ActionButton syntax:
+        ::
+
+            machine.mode.manual
+
+        """
         setTaskMode(linuxcnc.MODE_MANUAL)
 
     @staticmethod
     def auto():
+        """Change mode to Auto
+
+        ActionButton syntax:
+        ::
+
+            machine.mode.auto
+
+        """
         setTaskMode(linuxcnc.MODE_AUTO)
 
     @staticmethod
     def mdi():
+        """Change mode to MDI
+
+        ActionButton syntax:
+        ::
+
+            machine.mode.mdi
+
+        """
         setTaskMode(linuxcnc.MODE_MDI)
 
 def _mode_ok(widget=None):
@@ -536,16 +630,25 @@ class home:
     """Homing actions group"""
     @staticmethod
     def all():
-        """Homes all axes."""
+        """Homes all axes
+
+        ActionButton syntax::
+
+            machine.home.all
+        """
         LOG.info("Homing all axes")
         _home_joint(-1)
 
     @staticmethod
     def axis(axis):
-        """Home a specific axis.
+        """Home a specific axis
 
         Args:
-            axis (int | str) : Either the axis letter or number to home.
+            axis (int | str) : one of (xyzabcuvw or 012345678)
+
+        ActionButton syntax to home the X axis::
+
+            machine.home.axis:x
         """
         axis = getAxisLetter(axis)
         if axis.lower() == 'all':
@@ -557,10 +660,14 @@ class home:
 
     @staticmethod
     def joint(jnum):
-        """Home a specific joint.
+        """Home a specific joint
 
         Args:
-            jnum (int) : The number of the joint to home.
+            jnum (int) : one of (012345678)
+
+        ActionButton syntax to home joint 0::
+
+            machine.home.joint:0
         """
         LOG.info("Homing joint: {}".format(jnum))
         _home_joint(jnum)
@@ -584,15 +691,15 @@ def _home_ok(jnum=-1, widget=None):
     return ok
 
 def _home_all_bindOk(widget):
-    STATUS.on.connect(lambda: _home_ok(widget=widget))
-    STATUS.homed.onValueChanged(lambda: _home_ok(widget=widget))
+    STATUS.on.notify(lambda: _home_ok(widget=widget))
+    STATUS.homed.notify(lambda: _home_ok(widget=widget))
 
 home.all.ok = _home_ok
 home.all.bindOk = _home_all_bindOk
 
 def _home_joint_bindOk(jnum, widget):
-    STATUS.on.connect(lambda: _home_ok(jnum, widget=widget))
-    STATUS.homed.onValueChanged(lambda: _home_ok(jnum, widget=widget))
+    STATUS.on.notify(lambda: _home_ok(jnum, widget=widget))
+    STATUS.homed.notify(lambda: _home_ok(jnum, widget=widget))
 
 home.joint.ok = _home_ok
 home.joint.bindOk = _home_joint_bindOk
@@ -607,7 +714,7 @@ def _home_axis_bindOk(axis, widget):
         return
 
     jnum = INFO.AXIS_LETTER_LIST.index(axis)
-    STATUS.on.connect(lambda: _home_ok(jnum, widget=widget))
+    STATUS.on.notify(lambda: _home_ok(jnum, widget=widget))
 
 home.axis.ok = _home_ok
 home.axis.bindOk = _home_axis_bindOk
@@ -617,15 +724,47 @@ class unhome:
     """Unhoming actions group"""
     @staticmethod
     def all():
-        pass
+        """Unhome all the axes
+
+        ActionButton syntax::
+
+            machine.unhome.all
+        """
+        LOG.info("Unhoming all Axes")
+        _unhome_joint(-1)
 
     @staticmethod
     def axis(axis):
-        pass
+        """Unhome a specific axis
+
+        Args:
+            axis (int | str) : one of (xyzabcuvw or 012345678)
+
+        ActionButton syntax to unhome the X axis::
+
+            machine.unhome.axis:x
+        """
+        axis = getAxisLetter(axis)
+        if axis.lower() == 'all': # not sure what this is copied from home
+            unhome.all()
+            return
+        jnum = INFO.COORDINATES.index(axis)
+        LOG.info('Unhoming Axis: {}'.format(axis.upper()))
+        _unhome_joint(jnum)
 
     @staticmethod
     def joint(jnum):
-        pass
+        """Unhome a specific joint
+
+        Args:
+            jnum (int) : The number of the joint to home.
+
+        ActionButton syntax to unhome the joint 0::
+
+            machine.unhome.joint:0
+        """
+        LOG.info("Unhoming joint: {}".format(jnum))
+        _unhome_joint(jnum)
 
 def _home_joint(jnum):
     setTaskMode(linuxcnc.MODE_MANUAL)
@@ -635,7 +774,7 @@ def _home_joint(jnum):
 def _unhome_joint(jnum):
     setTaskMode(linuxcnc.MODE_MANUAL)
     CMD.teleop_enable(False)
-    CMD.home(jnum)
+    CMD.unhome(jnum)
 
 # Homing helper functions
 
@@ -675,9 +814,11 @@ def override_limits():
 
 def _override_limits_ok(widget=None):
     ok = False
+    mgs = None
     for anum in INFO.AXIS_NUMBER_LIST:
         if STAT.limit[anum] != 0:
-            aletter = 'XYZABCUVW'.index(anum)
+            aaxis = getAxisLetter(anum)
+            aletter = INFO.COORDINATES.index(aaxis)
             ok = True
             msg = "Axis {} on limit".format(aletter)
 
@@ -703,122 +844,83 @@ override_limits.bindOk = _override_limits_bindOk
 # JOG actions
 # -------------------------------------------------------------------------
 
-class jog:
+DEFAULT_JOG_SPEED = INFO.getJogVelocity()
+MAX_JOG_SPEED = INFO.getMaxJogVelocity()
 
-    max_linear_speed = INFO.getMaxJogVelocity()
+@setting('machine.jog.linear-speed',
+         default_value=DEFAULT_JOG_SPEED,
+         min_value=0,
+         max_value=MAX_JOG_SPEED,
+         persistent=True)
+def jog_linear_speed(obj):
+    return obj.value
 
-    linear_speed = INFO.getJogVelocity()
-    angular_speed = INFO.getJogVelocity()
-    continuous = False
-    increment = INFO.getIncrements()[0]
+@jog_linear_speed.setter
+def jog_linear_speed(obj, value):
 
-    @staticmethod
-    def axis(axis, direction=0, speed=None, distance=None):
-        """Jog an axis.
+    obj.value = obj.clampValue(value)
+    jog_linear_speed.signal.emit(obj.value)
+    LOG.debug("Setting jog linear speed: %4.1f", obj.value)
 
-        Args:
-            axis (str | int) : Either the letter or number of the axis to jog.
-                direction (str | int) : pos or +1 for positive, neg or -1 for negative.
-            speed (float, optional) : Desired jog vel in machine_units/s.
-                distance (float, optional) : Desired jog distance, continuous jog if 0.00.
-        """
-
-        if isinstance(direction, str):
-            direction = {'neg': -1, 'pos': 1}.get(direction.lower(), 0)
-
-        axis = getAxisNumber(axis)
-
-        # must be in teleoperating mode to jog.
-        # ToDo: probably need to do some checks to make sure it is OK to jog
-        if STAT.motion_mode != linuxcnc.TRAJ_MODE_TELEOP:
-            CMD.teleop_enable(1)
-
-        if speed == 0 or direction == 0:
-            CMD.jog(linuxcnc.JOG_STOP, 0, axis)
-
-        else:
-
-            if speed is None:
-                if axis in (3, 4, 5):
-                    speed = jog.angular_speed / 60
-                else:
-                    speed = jog.linear_speed / 60
-
-            if distance is None:
-                distance = jog.increment
-
-            velocity = float(speed) * direction
-
-            if distance == 0:
-                CMD.jog(linuxcnc.JOG_CONTINUOUS, 0, axis, velocity)
-            else:
-                CMD.jog(linuxcnc.JOG_INCREMENT, 0, axis, velocity, distance)
-
-    @staticmethod
-    def set_jog_continuous(continuous):
-        if continuous:
-            LOG.debug("Setting jog mode to continuous")
-        else:
-            LOG.debug("Setting jog mode to incremental")
-        jog.continuous = continuous
-
-    @staticmethod
-    def set_increment(raw_increment):
-        jog.increment = _parse_jog_increment(raw_increment)
-
-    @staticmethod
-    def set_linear_speed(speed):
-        jog.linear_speed = float(speed)
-
-    @staticmethod
-    def set_angular_speed(speed):
-        jog.angular_speed = float(speed)
+    percentage = int(obj.value * 100 / MAX_JOG_SPEED)
+    jog_linear_speed_percentage.value = percentage
+    jog_linear_speed_percentage.signal.emit(percentage)
 
 
-    @staticmethod
-    def set_linear_speed_percentage(percentage):
-        jog.set_linear_speed(jog.max_linear_speed * (float(percentage) / 100))
+@setting('machine.jog.linear-speed-percentage',
+         default_value=int(DEFAULT_JOG_SPEED * 100 / MAX_JOG_SPEED),
+         min_value=0,
+         max_value=100,
+         persistent=False)
+def jog_linear_speed_percentage(obj):
+    return obj.value
+
+@jog_linear_speed_percentage.setter
+def jog_linear_speed_percentage(obj, percentage):
+    LOG.debug("Setting jog linear speed percentage: %d", percentage)
+    jog_linear_speed.setValue(float(MAX_JOG_SPEED * percentage / 100))
 
 
-def _jog_speed_slider_bindOk(widget):
+@setting('machine.jog.angular-speed', default_value=100.0, persistent=False)
+def jog_angular_speed(obj):
+    return obj.value
 
-    try:
-        # these will only work for QSlider or QSpinBox
-        widget.setMinimum(0)
-        widget.setMaximum(100)
-        widget.setValue((jog.linear_speed / jog.max_linear_speed) * 100)
-
-        # jog.linear_speed.connect(lambda v: widget.setValue(v * 100))
-    except AttributeError:
-        pass
-
-jog.set_linear_speed.ok = jog.set_angular_speed.ok = lambda *a, **k: True
-jog.set_linear_speed.bindOk = jog.set_angular_speed.bindOk = lambda *a, **k: True
-
-jog.set_linear_speed_percentage.ok = lambda *a, **k: True
-jog.set_linear_speed_percentage.bindOk = _jog_speed_slider_bindOk
+@jog_angular_speed.setter
+def jog_angular_speed(obj, value):
+    LOG.debug("Setting Jog Angular Speed: %d", value)
+    obj.value = value
 
 
-def _from_internal_linear_unit(v, unit=None):
+@setting('machine.jog.mode-incremental', default_value=True)
+def jog_mode_incremental(obj):
+    return obj.value
+
+@jog_mode_incremental.setter
+def jog_mode_incremental(obj, value):
+    LOG.debug("Setting jog mode incremental: %s", value)
+    obj.value = value
+
+
+def fromInternalLinearUnits(v, unit=None):
     if unit is None:
         unit = STAT.linear_units
     lu = (unit or 1) * 25.4
     return v * lu
 
-def _parse_jog_increment(jogincr):
+def parseJogIncrement(jogincr):
     scale = 1
     if isinstance(jogincr, basestring):
         jogincr = jogincr.lower()
         if jogincr.endswith("mm"):
-            scale = _from_internal_linear_unit(1 / 25.4)
+            scale = fromInternalLinearUnits(1 / 25.4)
         elif jogincr.endswith("cm"):
-            scale = _from_internal_linear_unit(10 / 25.4)
+            scale = fromInternalLinearUnits(10 / 25.4)
         elif jogincr.endswith("um"):
-            scale = _from_internal_linear_unit(.001 / 25.4)
+            scale = fromInternalLinearUnits(.001 / 25.4)
         elif jogincr.endswith("in") or jogincr.endswith("inch") or jogincr.endswith('"'):
-            scale = _from_internal_linear_unit(1.)
+            scale = fromInternalLinearUnits(1.)
         elif jogincr.endswith("mil"):
-            scale = _from_internal_linear_unit(.001)
+            scale = fromInternalLinearUnits(.001)
         else:
             scale = 1
         jogincr = jogincr.rstrip(" inchmuil")
@@ -832,12 +934,146 @@ def _parse_jog_increment(jogincr):
             jogincr = 0
     return jogincr * scale
 
+@setting('machine.jog.increment', parseJogIncrement(INFO.getIncrements()[0]))
+def jog_increment(obj):
+    """Linear jog increment.
+
+    Args:
+        jogincr (str, int, float) : The desired jog increment. Can be passed
+            as a string including an optional units specifier. Valid unit
+            specifiers include ``mm``, ``cm``, ``um``, ``in``, ``inch``, ``"``,
+            and ``mil``.
+    """
+    return obj.value
+
+@jog_increment.setter
+def jog_increment(obj, jogincr):
+    value = parseJogIncrement(jogincr)
+    LOG.debug('Setting jog increment: %s (%2.4f)', jogincr, value)
+    obj.value = value
+    obj.signal.emit(value)
+
+
+class jog:
+    """Jog Actions Group"""
+
+    max_linear_speed = INFO.getMaxJogVelocity()
+    angular_speed = INFO.getJogVelocity()
+    continuous = False
+    increment = INFO.getIncrements()[0]
+
+    @staticmethod
+    def axis(axis, direction=0, speed=None, distance=None):
+        """Jog an axis.
+
+        Action Button Syntax to jog the X axis in the positive direction::
+
+            machine.jog.axis:x,pos
+
+        Args:
+            axis (str | int) : Either the letter or number of the axis to jog.
+            direction (str | int) : pos or +1 for positive, neg or -1 for negative.
+            speed (float, optional) : Desired jog vel in machine_units/s.
+            distance (float, optional) : Desired jog distance, continuous if 0.00.
+        """
+
+        # check if it even makes sense to try to jog
+        if STAT.task_state != linuxcnc.STATE_ON or STAT.task_mode != linuxcnc.MODE_MANUAL:
+            return
+
+        if isinstance(direction, str):
+            direction = {'neg': -1, 'pos': 1}.get(direction.lower(), 0)
+
+        axis = getAxisNumber(axis)
+
+        # must be in teleoperating mode to jog.
+        if STAT.motion_mode != linuxcnc.TRAJ_MODE_TELEOP:
+            CMD.teleop_enable(1)
+
+        if speed == 0 or direction == 0:
+            CMD.jog(linuxcnc.JOG_STOP, 0, axis)
+
+        else:
+
+            if speed is None:
+                if axis in (3, 4, 5):
+                    speed = jog_angular_speed.value / 60.0
+                else:
+                    speed = jog_linear_speed.value / 60.0
+
+            if distance is None:
+                if jog_mode_incremental.value:
+                    distance = jog_increment.value
+                else:
+                    distance = 0
+
+            velocity = float(speed) * direction
+
+            if distance == 0:
+                CMD.jog(linuxcnc.JOG_CONTINUOUS, 0, axis, velocity)
+            else:
+                CMD.jog(linuxcnc.JOG_INCREMENT, 0, axis, velocity, distance)
+
+    @staticmethod
+    def set_jog_continuous(continuous):
+        """Set Jog Continuous"""
+        if continuous:
+            LOG.debug("Setting jog mode to continuous")
+        else:
+            LOG.debug("Setting jog mode to incremental")
+        jog.jog_mode_incremental = continuous
+
+    @staticmethod
+    def set_increment(raw_increment):
+        """Set Jog Increment"""
+        jog_increment.setValue(raw_increment)
+
+    @staticmethod
+    def set_linear_speed(speed):
+        """Set Jog Linear Speed
+
+        ActionSlider syntax::
+
+            machine.jog.set-linear-speed
+        """
+        jog_linear_speed.setValue(float(speed))
+
+    @staticmethod
+    def set_angular_speed(speed):
+        """Set Jog Angular Speed"""
+        jog_angular_speed.setValue(float(speed))
+
+    @staticmethod
+    def set_linear_speed_percentage(percentage):
+        """Set Jog Linear Speed Percentage"""
+        jog_linear_speed_percentage.setValue(percentage)
+
+
+def _jog_speed_slider_bindOk(widget):
+
+    try:
+        # these will only work for QSlider or QSpinBox
+        widget.setMinimum(0)
+        widget.setMaximum(100)
+        widget.setValue((jog.linear_speed.getValue() / jog.max_linear_speed) * 100)
+
+        # jog.linear_speed.connect(lambda v: widget.setValue(v * 100))
+    except AttributeError:
+        pass
+
+jog.set_linear_speed.ok = jog.set_angular_speed.ok = lambda *a, **k: True
+jog.set_linear_speed.bindOk = jog.set_angular_speed.bindOk = lambda *a, **k: True
+
+jog.set_linear_speed_percentage.ok = lambda *a, **k: True
+jog.set_linear_speed_percentage.bindOk = _jog_speed_slider_bindOk
+
 
 def _jog_axis_ok(axis, direction=0, widget=None):
     axisnum = getAxisNumber(axis)
+    jnum = INFO.COORDINATES.index(axis)
     if STAT.task_state == linuxcnc.STATE_ON \
             and STAT.interp_state == linuxcnc.INTERP_IDLE \
-            and STAT.limit[axisnum] == 0:
+            and (STAT.limit[axisnum] == 0 or STAT.joint[jnum]['override_limits']):
         # and STAT.homed[axisnum] == 1 \
         ok = True
         msg = ""
@@ -874,22 +1110,38 @@ jog.axis.bindOk = _jog_axis_bindOk
 
 
 class jog_mode:
-
-    mode = 'continuous'
+    """Jog Mode Group"""
 
     @staticmethod
     def continuous():
-        LOG.debug("Setting jog mode continuous")
-        STATUS.setJogMode(True)
+        """Set Jog Continuous
+
+        ActionButton syntax::
+
+            machine.jog-mode.continuous
+        """
+        jog_mode_incremental.setValue(False)
+
 
     @staticmethod
     def incremental():
-        LOG.debug("Setting jog mode incremental")
-        STATUS.setJogMode(False)
+        """Set Jog Incremental
+
+        ActionButton syntax::
+
+            machine.jog-mode.incremental
+        """
+        jog_mode_incremental.setValue(True)
 
     @staticmethod
     def toggle():
-        if STATUS.jog_mode == True:
-            jog_mode.continuous()
-        else:
-            jog_mode.incremental()
\ No newline at end of file
+        """Jog Mode Toggle
+
+        ActionButton syntax::
+
+            machine.jog-mode.toggle
+        """
+        jog_mode_incremental.setValue(not jog_mode_incremental.value)
+
+jog_mode.incremental.ok = jog_mode.continuous.ok = lambda *a, **kw: True
+jog_mode.incremental.bindOk = jog_mode.continuous.bindOk = lambda *a, **kw: True
