@@ -41,13 +41,29 @@ from qtpy.QtWidgets import QApplication, QFileDialog
 from qtpyvcp.lib.types import DotDict
 from qtpyvcp.utilities.logger import initBaseLogger
 
+LCNC_VERSION_ERROR_MSG = """
+\033[31mERROR:\033[0m Unsupported LinuxCNC version
 
-if not linuxcnc.version.startswith('2.8'):
-    print "QtPyVCP only supports LinuxCNC 2.8, current version is", linuxcnc.version
-    print "If you have LinuxCNC installed as a RIP make sure you have\n" \
-          "activated the run-in-place environment by running:\n"
-    print "    $ . <linuxcnc-rip-dir>/scripts/rip-environment\n"
-    sys.exit()
+QtPyVCP only supports LinuxCNC 2.8, current version is {}.
+If you have LinuxCNC installed as a RIP make sure you have
+activated the run-in-place environment by running:\n"
+
+    $ . <linuxcnc-rip-dir>/scripts/rip-environment
+
+Otherwise you will need to install LinuxCNC 2.8, info on how
+to do that can be found here: https://gnipsel.com/linuxcnc/uspace/
+""".strip()
+
+INSTALLED_ERROR_MSG = """
+\033[31mERROR:\033[0m Can not edit an installed VCP
+
+The specified VCP appears to be installed in the `python2.7/site-packages` 
+directory. Please set up a development install to edit the VCP.
+""".strip()
+
+if linuxcnc.version.startswith('2.7'):
+    print(LCNC_VERSION_ERROR_MSG.format(linuxcnc.version))
+    sys.exit(1)
 
 LOG = initBaseLogger('qtpyvcp', log_file=os.devnull, log_level='WARNING')
 
@@ -61,7 +77,7 @@ def launch_designer(opts=DotDict()):
             options=QFileDialog.Options() | QFileDialog.DontUseNativeDialog)
 
         if not fname:
-            sys.exit()
+            sys.exit(1)
 
     else:
         fname = opts.vcp or opts.ui_file
@@ -80,11 +96,15 @@ def launch_designer(opts=DotDict()):
         except KeyError:
             pass
 
+        if 'lib/python2.7/site-packages' in fname:
+            print(INSTALLED_ERROR_MSG)
+            sys.exit(1)
+
     cmd = ['designer']
     ext = os.path.splitext(fname)[1]
     if ext in ['.yml', '.yaml']:
 
-        print "Loading YAML config file:", fname
+        print("Loading YAML config file:", fname)
 
         from qtpyvcp import CONFIG, DEFAULT_CONFIG_FILE
         from qtpyvcp.utilities.config_loader import load_config_files
@@ -93,41 +113,44 @@ def launch_designer(opts=DotDict()):
             CONFIG.update(load_config_files(fname, DEFAULT_CONFIG_FILE))
             data = CONFIG.get('qtdesigner')
         except:
-            print "Error loading YAML config file:"
+            print("Error loading YAML config file:")
             raise
 
+        from qtpyvcp.utilities.settings import addSetting
+        for k, v in CONFIG['settings'].items():
+            addSetting(k, **v)
+
         # add to path so that QtDesginer can load it when it starts
         os.environ['VCP_CONFIG_FILES'] = fname + ':' + os.getenv('VCP_CONFIG_FILES', '')
 
         if data is not None:
-            yml_dir = os.path.dirname(fname)
+            yml_dir = os.path.realpath(os.path.dirname(fname))
 
             # prefer command line ui file
             ui_file = opts.ui_file or data.get('ui_file')
             if ui_file is not None:
-                ui_file = os.path.realpath(os.path.join(yml_dir, ui_file))
+                ui_file = os.path.join(yml_dir, ui_file)
                 cmd.append(ui_file)
-                print "Loading UI file:", ui_file
+                print("Loading UI file:", ui_file)
             else:
-                print "No UI file specified."
+                print("No UI file specified.")
 
             # prefer command line qss file
             qss_file = opts.qss_file or data.get('qss_file')
             if qss_file is not None:
-                qss_file = os.path.realpath(os.path.join(yml_dir, qss_file))
+                qss_file = os.path.join(yml_dir, qss_file)
                 os.environ['QSS_STYLESHEET'] = qss_file
-                print "Loading QSS file:", qss_file
+                print("Loading QSS file:", qss_file)
             else:
-                print "No QSS file specified."
+                print("No QSS file specified.")
 
     elif ext == '.ui':
         cmd.append(fname)
 
-        print "Loading UI file:", fname
+        print("Loading UI file:", fname)
 
     else:
-        print "No valid file type selected.\n " \
-              "File must be a .yaml config file or a .ui file."
+        print("No valid file type selected.\n File must be a .yaml config file or a .ui file.")
         sys.exit()
 
     base = os.path.dirname(__file__)
@@ -137,8 +160,11 @@ def launch_designer(opts=DotDict()):
     os.environ['QT_SELECT'] = 'qt5'
     os.environ['PYQTDESIGNERPATH'] = os.path.join(base, '../widgets')
 
-    print "\nStarting QtDesigner ..."
-    sys.exit(subprocess.call(cmd))
+    print("\nStarting QtDesigner ...")
+    try:
+        sys.exit(subprocess.call(cmd))
+    except OSError:
+        print('Designer not found, Install with\nsudo apt install qttools5-dev qttools5-dev-tools')
 
 
 def main():
