@@ -1,4 +1,3 @@
-#!/usr/bin/env python
 """Utility for reading INI file info.
 
 This module is used to get information from the machine's INI file.
@@ -11,6 +10,9 @@ import sys
 
 from linuxcnc import ini
 
+from qtpyvcp import TOP_DIR
+from misc import normalizePath
+
 # Setup logging
 from qtpyvcp.utilities import logger
 log = logger.getLogger(__name__)
@@ -65,7 +67,7 @@ class _Info(object):
     def getMachineName(self):
         return self.ini.find('EMC', 'MACHINE') or "PyQtVCP Machine"
 
-    def getFilePath(self, section, option, base, default):
+    def getFilePath(self, section, option, base, default=None):
         path = self.ini.find(section, option) or default
         if path is None or not isinstance(path, str):
             return ""
@@ -97,9 +99,13 @@ class _Info(object):
         return self.getFilePath('EMCIO', 'TOOL_TABLE', self.CONFIG_DIR, default)
 
     def getPostguiHalfile(self):
-        return self.ini.find("HAL", "POSTGUI_HALFILE")
+        return self.getFilePath("HAL", "POSTGUI_HALFILE", self.CONFIG_DIR)
 
     def getOpenFile(self, default=None):
+        if default is None:
+            default = os.path.realpath(os.path.join(TOP_DIR, 'sim/example_gcode/qtpyvcp.ngc'))
+            if not os.path.isfile(default):
+                default = os.path.expanduser('~/linuxcnc/nc_files/.qtpyvcp/qtpyvcp.ngc')
         return self.getFilePath('DISPLAY', 'OPEN_FILE', self.CONFIG_DIR, default)
 
     def getCoordinates(self):
@@ -167,6 +173,9 @@ class _Info(object):
             return False
         return True
 
+    def getEditor(self):
+        return self.ini.find('DISPLAY', 'EDITOR') or 'xdg-open'
+
     def getJogVelocity(self):
         # get default jog velocity
         # must convert from self.ini's units per second to hazzys's units per minute
@@ -232,7 +241,7 @@ class _Info(object):
     def getProgramPrefix(self):
         path = self.ini.find('DISPLAY', 'PROGRAM_PREFIX')
         if path:
-            path = os.path.expanduser(path)
+            path = normalizePath(path, os.getenv("CONFIG_DIR"))
             if os.path.exists(path):
                 return path
             else:
@@ -305,13 +314,23 @@ class _Info(object):
         return jog_increments
 
     def getSubroutinePath(self):
-        subroutines_path = self.ini.find('RS274NGC', 'SUBROUTINE_PATH')
-        if not subroutines_path:
-            log.info("No subroutine folder or program prefix given in self.ini file")
-            subroutines_path = self.getProgramPrefix()
-        if not subroutines_path:
-            return False
-        return subroutines_path
+        temp = self.ini.find('RS274NGC', 'SUBROUTINE_PATH')
+        if temp is None:
+            log.info("No subroutine path specified in INI file")
+            return self.getProgramPrefix()
+        return temp
+
+    def getSubroutineSearchDirs(self):
+        config_dir = os.getenv("CONFIG_DIR", "")
+        dirs = [self.getProgramPrefix(),]
+        temp = self.ini.find('RS274NGC', 'SUBROUTINE_PATH')
+        if temp is None:
+            log.info("No subroutine path specified in INI file")
+        else:
+            for path in temp.strip(':').split(':'):
+                dirs.append(normalizePath(path, config_dir))
+
+        return dirs
 
     def getRS274StartCode(self):
         temp = self.ini.find('RS274NGC', 'RS274NGC_STARTUP_CODE')
@@ -375,7 +394,7 @@ class _Info(object):
                     self.ALETTER_JNUM_DICT[aletter] = jnum
                     log.info("Axis {0} --> Joint {1}".format(aletter, jnum))
 
-        # print "AXIS_LETTER_LIST ", AXIS_LETTER_LIST
-        # print "JOINT_AXIS_DICT, ", JOINT_AXIS_DICT
-        # print "ALETTER_JNUM_DICT ", ALETTER_JNUM_DICT
-        # print "JNUM_ALETTER_DICT ", JNUM_ALETTER_DICT
+        # print("AXIS_LETTER_LIST ", AXIS_LETTER_LIST)
+        # print("JOINT_AXIS_DICT, ", JOINT_AXIS_DICT)
+        # print("ALETTER_JNUM_DICT ", ALETTER_JNUM_DICT)
+        # print("JNUM_ALETTER_DICT ", JNUM_ALETTER_DICT)
