@@ -1,5 +1,3 @@
-#!/usr/bin/env python
-
 import os
 
 def normalizePath(path, base):
@@ -20,7 +18,7 @@ def insertPath(env_var, index, file):
         files =[]
     else:
         files.strip(':').split(':')
-    print files
+    print(files)
     files.insert(index, file)
     os.environ[env_var] = ':'.join(files)
-    print os.environ[env_var]
+    print(os.environ[env_var])
