@@ -36,6 +36,17 @@ class VCPChooser(QDialog):
             child = QTreeWidgetItem(category)
             child.setText(0, self.get_vcp_data(entry_point))
 
+        # test VCP section
+        category = QTreeWidgetItem(self.vcpTreeView)
+        category.setText(0, 'Video Test VCPs')
+        category.setFlags(Qt.ItemIsEnabled)
+
+        # add example VCPs to the treeview
+        for entry_point in iter_entry_points(group='qtpyvcp.test_vcp'):
+            child = QTreeWidgetItem(category)
+            child.setText(0, self.get_vcp_data(entry_point))
+
+
         # installed VCP section
         category = QTreeWidgetItem(self.vcpTreeView)
         category.setText(0, 'Installed VCPs')
@@ -64,10 +75,10 @@ class VCPChooser(QDialog):
 
     def get_vcp_data(self, entry_point):
         vcp_data = {'entry_point_name': entry_point.name}
-        try:
-            vcp = entry_point.load()
-            vcp_config_file = vcp.VCP_CONFIG_FILE
+        vcp = entry_point.load()
+        vcp_config_file = vcp.VCP_CONFIG_FILE
 
+        if os.path.exists(vcp_config_file):
             with open(vcp_config_file, 'r') as fh:
                 lines = fh.readlines()
 
@@ -78,10 +89,11 @@ class VCPChooser(QDialog):
                     continue
                 clean.append(line)
 
-            vcp_data.update(yaml.load(''.join(clean)).get('vcp', {}))
+            config = yaml.load(''.join(clean), Loader=yaml.SafeLoader)
+            if config is not None:
+                vcp_data.update(config.get('vcp', {}))
             vcp_name = vcp_data.get('name', entry_point.name)
-
-        except:
+        else:
             vcp_name = entry_point.name
 
         self._vcp_data[vcp_name] = vcp_data
