@@ -1,14 +1,12 @@
-#!/usr/bin/env python
-
 import os
 import re
-from qtpy.QtWidgets import QPushButton, qApp
-from qtpy.QtCore import Qt, QEvent, Property
+from qtpy.QtWidgets import qApp
+from qtpy.QtCore import Property
 
-from qtpyvcp.core import Info
+from qtpyvcp.utilities.info import Info
 INFO = Info()
 
-from qtpyvcp.widgets import CMDWidget
+from qtpyvcp.widgets import VCPButton
 from qtpyvcp.actions.machine_actions import issue_mdi
 
 from qtpyvcp.utilities import logger
@@ -19,19 +17,31 @@ LOG = logger.getLogger(__name__)
 # if a group is not present it will be an empty string
 PARSE_POSITIONAL_ARGS = re.compile(r' *# *<([a-z0-9_-]+)> *= *#([0-9]+) *(?:\(= *([0-9.+-]+[0-9.]*?|) *(.*)\))?', re.I)
 
-SUBROUTINE_PATH = INFO.getSubroutinePath()
+SUBROUTINE_SEARCH_DIRS = INFO.getSubroutineSearchDirs()
 
-class SubCallButton(QPushButton, CMDWidget):
+class SubCallButton(VCPButton):
     """Button for calling ngc subroutines.
 
     Args:
         parent (QWidget, optional) : The parent widget of the button, or None.
         filename (str, optional) : The filename of the NGCGUI style subroutine
             the button should call, including any extension. The subroutine must
-            be on the subroutine path specified in the INI.
+            be on the subroutine path specified in the INI. The name of the
+            subroutine must match exactly the sub/endsub name. The parameter
+            #<parameter1> if found in the VCP the value from that widget will be
+            used instead of the default value. If you don't have a default value
+            you must have a widget by the same name. The widget can be a line
+            edit, a spin box or a double spin box.
+    ::
 
-    Todo:
-        Add NGCGUI subroutine file format example.
+        example.ngc
+        o<example> sub
+        #<parameter1> = #1
+        #<parameter2> = #2 (=default_value)
+
+        ;Body of the subroutine
+
+        o<example> endsub
     """
     def __init__(self, parent=None, filename=''):
         super(SubCallButton, self).__init__(parent)
@@ -44,9 +54,14 @@ class SubCallButton(QPushButton, CMDWidget):
     def callSub(self):
         window = qApp.activeWindow()
 
-        subfile = os.path.join(SUBROUTINE_PATH, self._filename)
-        if not os.path.isfile(subfile):
-            LOG.error('Subroutine file does not exist: yellow<{}>'.format(subfile))
+        subfile = None
+        for dir in SUBROUTINE_SEARCH_DIRS:
+            subfile = os.path.join(dir, self._filename)
+            if os.path.isfile(subfile):
+                break
+
+        if subfile is None:
+            LOG.error('Subroutine file could not be found: yellow<{}>'.format(self._filename))
             return False
 
         with open(subfile, 'r') as fh:
