@@ -1,31 +1,30 @@
-#!/usr/bin/env python
-# coding: utf-8
-
 import os
 import sys
 
 from qtpy import uic
+from qtpy.QtGui import QKeySequence
 from qtpy.QtCore import Qt, Slot, QTimer
-from qtpy.QtWidgets import QMainWindow, QApplication, QAction, QMessageBox, QMenu, QMenuBar, QLineEdit
+from qtpy.QtWidgets import QMainWindow, QApplication, QAction, QMessageBox, \
+    QMenu, QMenuBar, QLineEdit, QShortcut, QActionGroup
 
 import qtpyvcp
 from qtpyvcp import actions
 from qtpyvcp.utilities import logger
-from qtpyvcp.core import Prefs, Info
-from qtpyvcp.lib.types import DotDict
-from qtpyvcp.widgets.dialogs import showDialog
+from qtpyvcp.utilities.info import Info
 from qtpyvcp.plugins import getPlugin
-from qtpyvcp.vcp_launcher import _initialize_object_from_dict
+from qtpyvcp.utilities.settings import getSetting
+from qtpyvcp.widgets.dialogs import showDialog as _showDialog
+from qtpyvcp.app.launcher import _initialize_object_from_dict
 
 LOG = logger.getLogger(__name__)
-PREFS = Prefs()
 INFO = Info()
 
 
 class VCPMainWindow(QMainWindow):
 
     def __init__(self, parent=None, opts=None, ui_file=None, stylesheet=None,
-                 position=None, size=None, confirm_exit=True, title=None, menu=None):
+                 maximize=False, fullscreen=False, position=None, size=None,
+                 confirm_exit=True, title=None, menu='default'):
 
         super(VCPMainWindow, self).__init__(parent)
 
@@ -35,7 +34,6 @@ class VCPMainWindow(QMainWindow):
         self.setWindowTitle(title)
 
         self.app = QApplication.instance()
-        self.status = getPlugin('status')
 
         self.confirm_exit = confirm_exit if opts.confirm_exit is None else opts.confirm_exit
 
@@ -53,6 +51,9 @@ class VCPMainWindow(QMainWindow):
             except AttributeError:
                 pass
 
+            if menu == 'default':
+                menu = qtpyvcp.CONFIG.get('default_menubar', [])
+
             self.setMenuBar(self.buildMenuBar(menu))
 
         if title is not None:
@@ -61,30 +62,34 @@ class VCPMainWindow(QMainWindow):
         if stylesheet is not None:
             self.loadStylesheet(stylesheet)
 
-        if opts.maximize:
+        maximize = opts.maximize if opts.maximize is not None else maximize
+        if maximize:
             QTimer.singleShot(0, self.showMaximized)
 
-        if opts.fullscreen:
+        fullscreen = opts.fullscreen if opts.fullscreen is not None else fullscreen
+        if fullscreen:
             QTimer.singleShot(0, self.showFullScreen)
 
         if opts.hide_menu_bar:
             self.menuBar().hide()
 
-        if opts.size:
+        size = opts.size or size
+        if size:
             try:
-                width, height = opts.size.lower().split('x')
+                width, height = size.lower().split('x')
                 self.resize(int(width), int(height))
             except:
-                LOG.exception('Error parsing --size argument: %s', opts.size)
+                LOG.exception('Error parsing --size argument: %s', size)
 
-        if opts.position:
+        pos = opts.position or position
+        if pos:
             try:
-                xpos, ypos = opts.position.lower().split('x')
+                xpos, ypos = pos.lower().split('x')
                 self.move(int(xpos), int(ypos))
             except:
-                LOG.exception('Error parsing --position argument: %s', opts.position)
+                LOG.exception('Error parsing --position argument: %s', pos)
 
-        # QShortcut(QKeySequence("t"), self, self.test)
+        QShortcut(QKeySequence("F11"), self, self.toggleFullscreen)
         self.app.focusChanged.connect(self.focusChangedEvent)
 
     def loadUi(self, ui_file):
@@ -104,35 +109,93 @@ class VCPMainWindow(QMainWindow):
             stylesheet (str) : Path to a .qss stylesheet
         """
         LOG.info("Loading QSS stylesheet file: yellow<{}>".format(stylesheet))
-        with open(stylesheet, 'r') as fh:
-            self.setStyleSheet(fh.read())
+        self.setStyleSheet("file:///" + stylesheet)
 
-    def getMenuAction(self, menu_action, title='notitle', action_name='noaction'):
+    def getMenuAction(self, menu, title='notitle', action_name='noaction',
+                      shortcut="", args=[], kwargs={}):
         # ToDo: Clean this up, it is very hacky
         env = {'app': QApplication.instance(),
                'win': self,
                'action': actions,
                }
 
-        try:
-            mod, action = action_name.split('.', 1)
-            method = getattr(env.get(mod, self), action)
-            menu_action.triggered.connect(method)
-            return
-        except:
-            pass
+        if action_name is not None:
 
-        try:
-            actions.bindWidget(menu_action, action_name)
-            return
-        except actions.InvalidAction:
-            pass
+            if action_name.startswith('settings.'):
+                setting_id = action_name[len('settings.'):]
+                setting = getSetting(setting_id)
 
+                if setting:
+                    if setting.enum_options is not None:
+                        submenu = QMenu(parent=self, title=title)
+
+                        group = QActionGroup(self)
+                        group.setExclusive(True)
+                        group.triggered.connect(lambda a: setting.setValue(a.data()))
+
+                        def update(group, val):
+                            for act in group.actions():
+                                if act.data() == val:
+                                    act.setChecked(True)
+                                    break
+
+                        for num, opt in enumerate(setting.enum_options):
+
+                            menu_action = QAction(parent=self, text=opt)
+                            menu_action.setCheckable(True)
+                            if setting.value == num:
+                                menu_action.setChecked(True)
+
+                            menu_action.setData(num)
+                            setting.notify(lambda v: update(group, v))
+
+                            menu_action.setActionGroup(group)
+                            submenu.addAction(menu_action)
+                        menu.addMenu(submenu)
+
+                    elif setting.value_type == bool:
+                        # works for bool settings
+                        menu_action = QAction(parent=self, text=title)
+                        menu_action.setCheckable(True)
+                        menu_action.triggered.connect(setting.setValue)
+                        menu_action.setShortcut(shortcut)
+                        setting.notify(menu_action.setChecked)
+                        menu.addAction(menu_action)
+
+                    return
+
+            try:
+                menu_action = QAction(parent=self, text=title)
+
+                mod, action = action_name.split('.', 1)
+                method = getattr(env.get(mod, self), action)
+                if menu_action.isCheckable():
+                    menu_action.triggered.connect(method)
+                else:
+                    menu_action.triggered.connect(lambda checked: method(*args, **kwargs))
+
+                menu_action.setShortcut(shortcut)
+                menu.addAction(menu_action)
+                return
+            except:
+                pass
+
+            try:
+                menu_action = QAction(parent=self, text=title)
+                actions.bindWidget(menu_action, action_name)
+                menu_action.setShortcut(shortcut)
+                menu.addAction(menu_action)
+                return
+            except actions.InvalidAction:
+                LOG.exception('Error binding menu action %s', action_name)
+
+        menu_action = QAction(parent=self, text=title)
         msg = "The <b>{}</b> action specified for the " \
               "<b>{}</b> menu item could not be triggered. " \
               "Check the YAML config file for errors." \
               .format(action_name or '', title.replace('&', ''))
         menu_action.triggered.connect(lambda: QMessageBox.critical(self, "Menu Action Error!", msg))
+        menu.addAction(menu_action)
 
     def buildMenuBar(self, menus):
         """Recursively build menu bar.
@@ -160,6 +223,8 @@ class VCPMainWindow(QMainWindow):
                 title = item.get('title') or ''
                 items = item.get('items')
                 provider = item.get('provider')
+                args = item.get('args') or []
+                kwargs = item.get('kwargs') or {}
 
                 if items is not None:
                     new_menu = QMenu(parent=self, title=title)
@@ -172,19 +237,24 @@ class VCPMainWindow(QMainWindow):
                     menu.addMenu(new_menu)
 
                 else:
-                    act = QAction(parent=self, text=title)
-                    self.getMenuAction(act, title, item.get('action'))
-                    act.setShortcut(item.get('shortcut', ''))
-                    menu.addAction(act)
+                    self.getMenuAction(menu, title, item.get('action'),
+                                       item.get('shortcut', ''),
+                                       item.get('args', []),
+                                       item.get('kwargs', {}))
 
         menu_bar = QMenuBar(self)
         recursiveAddItems(menu_bar, menus)
         return menu_bar
 
     def initUi(self):
-        self.status.init_ui.emit()
         self.loadSplashGcode()
-        self.initHomingMenu()
+
+    @Slot()
+    def toggleFullscreen(self):
+        if self.isFullScreen():
+            self.showNormal()
+        else:
+            self.showFullScreen()
 
     def closeEvent(self, event):
         """Catch close event and show confirmation dialog."""
@@ -207,20 +277,21 @@ class VCPMainWindow(QMainWindow):
         if event.isAutoRepeat():
             return
 
+        speed = actions.machine.MAX_JOG_SPEED / 60.0 if event.modifiers() & Qt.ShiftModifier else None
         if event.key() == Qt.Key_Up:
-            actions.machine.jog.axis('Y', 1)
+            actions.machine.jog.axis('Y', 1, speed=speed)
         elif event.key() == Qt.Key_Down:
-            actions.machine.jog.axis('Y', -1)
+            actions.machine.jog.axis('Y', -1, speed=speed)
         elif event.key() == Qt.Key_Left:
-            actions.machine.jog.axis('X', -1)
+            actions.machine.jog.axis('X', -1, speed=speed)
         elif event.key() == Qt.Key_Right:
-            actions.machine.jog.axis('X', 1)
+            actions.machine.jog.axis('X', 1, speed=speed)
         elif event.key() == Qt.Key_PageUp:
-            actions.machine.jog.axis('Z', 1)
+            actions.machine.jog.axis('Z', 1, speed=speed)
         elif event.key() == Qt.Key_PageDown:
-            actions.machine.jog.axis('Z', -1)
-        else:
-            print 'Unhandled key press event'
+            actions.machine.jog.axis('Z', -1, speed=speed)
+        #else:
+            #print('Unhandled key press event')
 
     def keyReleaseEvent(self, event):
         if event.isAutoRepeat():
@@ -238,18 +309,19 @@ class VCPMainWindow(QMainWindow):
             actions.machine.jog.axis('Z', 0)
         elif event.key() == Qt.Key_PageDown:
             actions.machine.jog.axis('Z', 0)
-        else:
-            print 'Unhandled key release event'
+        #else:
+            #print('Unhandled key release event')
 
     def mousePressEvent(self, event):
-        print 'Button press'
+        #print('Button press')
         focused_widget = self.focusWidget()
         if focused_widget is not None:
             focused_widget.clearFocus()
 
     def focusChangedEvent(self, old_w, new_w):
         if issubclass(new_w.__class__, QLineEdit):
-            print "QLineEdit got focus: ", new_w
+            #print("QLineEdit got focus: ", new_w)
+            QTimer.singleShot(0, new_w.selectAll)
 
 # ==============================================================================
 #  menu action slots
@@ -257,39 +329,17 @@ class VCPMainWindow(QMainWindow):
 
     @Slot()
     def openFile(self):
-        showDialog('open_file')
+        _showDialog('open_file')
 
+    @Slot(str)
     def showDialog(self, dialog_name):
-        showDialog(dialog_name)
+        _showDialog(dialog_name)
 
 # ==============================================================================
 # menu functions
 # ==============================================================================
 
-    def initHomingMenu(self):
-        if hasattr(self, 'menuHoming'):
 
-            # remove any actions that were added in QtDesigner
-            for menu_action in self.menuHoming.actions():
-                self.menuHoming.removeAction(menu_action)
-
-            # Register the submenu with the action (so it will be disabled
-            # if the actions are not valid), but don't connect it to method
-            # home_action = action.Home(widget=self.menuHoming, method=None)
-            # FIXME:
-            # home_action = actions.bindWidget(self.menuHoming, 'machine.home.all.ok')
-
-            menu_action = QAction(self)
-            menu_action.setText("Home &All")
-            actions.bindWidget(menu_action, 'machine.home.all')
-            self.menuHoming.addAction(menu_action)
-
-            # add homing actions for each axis
-            for aletter in INFO.AXIS_LETTER_LIST:
-                menu_action = QAction(self)
-                menu_action.setText("Home &{}".format(aletter.upper()))
-                actions.bindWidget(menu_action, 'machine.home.axis:{}'.format(aletter))
-                self.menuHoming.addAction(menu_action)
 
 # ==============================================================================
 # helper functions
@@ -297,9 +347,9 @@ class VCPMainWindow(QMainWindow):
 
     def loadSplashGcode(self):
         # Load backplot splash code
-        path = os.path.realpath(os.path.join(__file__, '../../../..', 'sim/example_gcode/qtpyvcp.ngc'))
-        splash_code = INFO.getOpenFile() or path
-        if splash_code is not None:
+        splash_code = INFO.getOpenFile()
+        #print(splash_code)
+        if splash_code is not None and os.path.isfile(splash_code):
             # Load after startup to not cause hang and 'Can't set mode while machine is running' error
             QTimer.singleShot(200, lambda: actions.program.load(splash_code, add_to_recents=False))
 
