@@ -1,15 +1,26 @@
+# Copyright (c) 2017-2018, SLAC National Accelerator Laboratory
+
+# This file has been adapted from PyDM, and can be redistributed and/or
+# modified in accordance with terms in conditions set forth in the BSD
+# 3-Clause License. You can find the complete licence text in the LICENCES
+# directory.
+
+# Links:
+#   PyDM Project: https://github.com/slaclab/pydm
+#   PyDM Licence: https://github.com/slaclab/pydm/blob/master/LICENSE.md
+
 import os
 import json
 import functools
 import webbrowser
 
+from qtpy import uic
 from qtpy import QtWidgets, QtCore, QtDesigner
 
-import qtpyvcp
-from qtpyvcp.plugins import QtPyVCPDataChannel
+from qtpyvcp.plugins import DataPlugin, DataChannel, getPlugin, iterPlugins
+from qtpyvcp.utilities.settings import Setting
 from plugin_extension import _PluginExtension
 
-
 # Set up logging
 from qtpyvcp.utilities import logger
 LOG = logger.getLogger(__name__)
@@ -22,6 +33,7 @@ RULE_PROPERTIES = {
     # 'Opacity': ['setOpacity', float]
 }
 
+
 class RulesEditorExtension(_PluginExtension):
     def __init__(self, widget):
         super(RulesEditorExtension, self).__init__(widget)
@@ -31,6 +43,36 @@ class RulesEditorExtension(_PluginExtension):
     def editAction(self, state):
         RulesEditor(self.widget, parent=None).exec_()
 
+
+class ChanInfoDialog(QtWidgets.QDialog):
+    def __init__(self, info, parent=None):
+        super(ChanInfoDialog, self).__init__(parent)
+        ui_file = os.path.join(os.path.dirname(os.path.abspath(__file__)),
+                               "channel_info_dialog.ui")
+        uic.loadUi(ui_file, self)
+
+        ch_obj, ch_exp, ch_val, ch_doc = info
+
+        doc_lines = [l.strip() for l in ch_doc.strip().splitlines()]
+        title = doc_lines[0]
+
+        self.lbl_rtn_typ.setText(type(ch_obj.getValue()).__name__)
+
+        for line in doc_lines:
+            if line.startswith(':returns:'):
+                self.lbl_chan_rtn.setText(line.replace(':returns:', '').strip())
+            elif line.startswith(':rtype:'):
+                self.lbl_rtn_typ.setText(line.replace(':rtype:', '').strip())
+
+        self.lbl_chan_name.setText(title)
+        self.tb_chan_doc.setText(ch_doc)
+        val = ch_obj.getValue()
+        txt = ''
+        if len(self.lbl_rtn_typ.text().split(',')) > 1:
+            txt = ', ' + str(ch_val)
+        self.lbl_can_val.setText(str(val) + txt)
+
+
 class TableCheckButton(QtWidgets.QWidget):
     def __init__(self, checked=False):
         super(TableCheckButton, self).__init__()
@@ -45,6 +87,29 @@ class TableCheckButton(QtWidgets.QWidget):
     def __getattr__(self, attr):
         return getattr(self.chk_bx, attr)
 
+
+class CompleterDelegate(QtWidgets.QStyledItemDelegate):
+    def __init__(self, parent=None):
+        super(CompleterDelegate, self).__init__(parent)
+
+        items = []
+        for plugin, obj in iterPlugins():
+            if isinstance(obj, DataPlugin):
+                for chan_name in obj.channels:
+                    items.append('{}:{}'.format(plugin, chan_name))
+
+        self.completer = QtWidgets.QCompleter(sorted(items))
+        self.completer.setCompletionColumn(0)
+        self.completer.setCompletionRole(QtCore.Qt.EditRole)
+        self.completer.setCaseSensitivity(QtCore.Qt.CaseInsensitive)
+
+    def createEditor(self, parent, option, index):
+        editor = QtWidgets.QLineEdit(parent)
+        editor.setFrame(False)
+        editor.setCompleter(self.completer)
+        return editor
+
+
 class RulesEditor(QtWidgets.QDialog):
     """QDialog for user-friendly editing of widget Rules in Qt Designer.
 
@@ -180,6 +245,7 @@ class RulesEditor(QtWidgets.QDialog):
         self.cmb_property = QtWidgets.QComboBox()
         for name, prop in self.available_properties.items():
             self.cmb_property.addItem(name, prop)
+        self.cmb_property.model().sort(0)
         edit_name_layout.addRow(lbl_property, self.cmb_property)
 
         frm_edit_layout.addLayout(edit_name_layout)
@@ -206,6 +272,8 @@ class RulesEditor(QtWidgets.QDialog):
         self.tbl_channels.setMinimumWidth(350)
         self.tbl_channels.setShowGrid(True)
         self.tbl_channels.setCornerButtonEnabled(False)
+        delegate = CompleterDelegate(self.tbl_channels)
+        self.tbl_channels.setItemDelegateForColumn(0, delegate)
         self.tbl_channels.model().dataChanged.connect(self.tbl_channels_changed)
         headers = ["Channel", "Trigger", "Type"]
         self.tbl_channels.setColumnCount(len(headers))
@@ -214,7 +282,8 @@ class RulesEditor(QtWidgets.QDialog):
         header.setSectionResizeMode(0, QtWidgets.QHeaderView.Stretch)
         header.setSectionResizeMode(1, QtWidgets.QHeaderView.ResizeToContents)
         header.setSectionResizeMode(2, QtWidgets.QHeaderView.ResizeToContents)
-        # self.tbl_channels.setColumnWidth(2, 60)
+        # header.setSectionResizeMode(3, QtWidgets.QHeaderView.ResizeToContents)
+        self.tbl_channels.setColumnWidth(3, 8)
 
         frm_edit_layout.addWidget(self.tbl_channels)
 
@@ -224,7 +293,16 @@ class RulesEditor(QtWidgets.QDialog):
         self.lbl_expected_type = QtWidgets.QLabel(parent=self)
         # self.lbl_expected_type.setText("")
         self.lbl_expected_type.setStyleSheet("color: rgb(0, 128, 255); font-weight: bold;")
-        expression_layout.addRow(lbl_expected, self.lbl_expected_type)
+
+        hbox = QtWidgets.QHBoxLayout()
+        hbox.addWidget(self.lbl_expected_type)
+        hbox.addStretch()
+        self.btn_chan_info = QtWidgets.QPushButton("Info ...")
+        self.btn_chan_info.setAutoDefault(False)
+        self.btn_chan_info.pressed.connect(self.show_chan_info)
+        hbox.addWidget(self.btn_chan_info)
+
+        expression_layout.addRow(lbl_expected, hbox)
 
         lbl_expression = QtWidgets.QLabel("Expression:")
         expr_help_layout = QtWidgets.QHBoxLayout()
@@ -277,7 +355,7 @@ class RulesEditor(QtWidgets.QDialog):
         for row, ch in enumerate(channels):
             ch_name = ch.get('url', '')
             ch_tr = ch.get('trigger', False)
-            ch_obj, ch_val, ch_desc = self.get_channel_data(ch_name)
+            ch_obj, ch_exp, ch_val, ch_desc = self.get_channel_data(ch_name)
             self.tbl_channels.setItem(row, 0,
                                       QtWidgets.QTableWidgetItem(str(ch_name)))
 
@@ -286,14 +364,13 @@ class RulesEditor(QtWidgets.QDialog):
 
             typ_lbl = QtWidgets.QLabel()
             typ_lbl.setAlignment(QtCore.Qt.AlignCenter)
+
             if ch_val is None:
                 typ_lbl.setText("<font color='red'>error</font>")
             else:
                 typ_lbl.setText("<font color='green'>{}</font>".format(type(ch_val).__name__))
             self.tbl_channels.setCellWidget(row, 2, typ_lbl)
 
-            # self.tbl_channels.setItem(row, 2, QtWidgets.QTableWidgetItem(type(ch_val).__name__))
-
         self.frm_edit.setEnabled(True)
         self.loading_data = False
 
@@ -419,6 +496,15 @@ class RulesEditor(QtWidgets.QDialog):
         else:
             return help_url
 
+    def show_chan_info(self):
+        selection = self.tbl_channels.selectedIndexes()
+        if len(selection) > 0:
+            row = selection[0].row()
+            model = self.tbl_channels.model()
+            chan = model.data(model.index(row, 0))
+            info = self.get_channel_data(chan)
+            ChanInfoDialog(info).exec_()
+
     def name_changed(self):
         """Callback executed when the rule name is changed."""
         self.lst_rule_item.setText(self.txt_name.text())
@@ -426,7 +512,6 @@ class RulesEditor(QtWidgets.QDialog):
 
     def property_changed(self, index):
         """Callback executed when the property is selected."""
-        print "prop changed"
         try:
             prop = self.cmb_property.currentData()
             if prop[1] is None:
@@ -439,7 +524,7 @@ class RulesEditor(QtWidgets.QDialog):
             idx = self.get_current_index()
             self.change_entry("property", self.cmb_property.currentText())
         except Exception as e:
-            print "error", e
+            print("error", e)
             self.lbl_expected_type.setText("")
 
     def tbl_channels_changed(self, table_item):
@@ -449,13 +534,15 @@ class RulesEditor(QtWidgets.QDialog):
 
         row = table_item.row()
         ch_name = table_item.data()
-        ch_obj, ch_val, ch_desc = self.get_channel_data(ch_name)
+        ch_obj, ch_exp, ch_val, ch_desc = self.get_channel_data(ch_name)
 
         typ_lbl = self.tbl_channels.cellWidget(row, 2)
-        if ch_val is None:
-            typ_lbl.setText("<font color='red'>error</font>")
+
+        if isinstance(ch_obj, (DataChannel, Setting)) and ch_val is not None:
+            typ_lbl.setText("<font color='green'>{}</font>"
+                            .format(type(ch_val).__name__))
         else:
-            typ_lbl.setText("<font color='green'>{}</font>".format(type(ch_val).__name__))
+            typ_lbl.setText("<font color='red'>error</font>")
 
         self.update_channels()
 
@@ -472,27 +559,20 @@ class RulesEditor(QtWidgets.QDialog):
         self.change_entry("channels", new_channels)
 
     def get_channel_data(self, url):
-        chan_obj = None
-        chan_val = None
-        chan_desc = ''
 
+        protocol, sep, item = url.partition(':')
         try:
-            protocol, sep, rest = url.partition(':')
-            item, sep, query = rest.partition('?')
+            plugin = getPlugin(protocol)
+        except ValueError:
+            return None, None, None, None
 
-            if query == '':
-                query = 'value'
+        chan_obj, chan_exp = plugin.getChannel(item)
+        if chan_obj is not None:
+            chan_val = chan_exp()
+        else:
+            chan_val = None
 
-            plugin = qtpyvcp.PLUGINS[protocol]
-            eval_env = {'plugin': plugin}
-
-            chan_obj = eval("plugin.{}".format(item), eval_env)
-            chan_val = chan_obj.handleQuery(query)
-
-        except:
-            LOG.exception("Error in eval")
-
-        return chan_obj, chan_val, chan_desc
+        return chan_obj, chan_exp, chan_val, chan_obj.__doc__
 
     def expression_changed(self):
         """Callback executed when the expression is modified."""
@@ -538,11 +618,10 @@ class RulesEditor(QtWidgets.QDialog):
                         found_trigger = True
 
                     # get chan values for use when checking expression
-                    ch_obj, ch_val, ch_desc = self.get_channel_data(ch.get("url", ""))
-                    print ch_obj
+                    ch_obj, ch_exp, ch_val, ch_desc = self.get_channel_data(ch.get("url", ""))
 
-                    if isinstance(ch_obj, QtPyVCPDataChannel):
-                        channel_values.append(ch_val)
+                    if isinstance(ch_obj, (DataChannel, Setting)):
+                        channel_values.append(ch_exp())
                     else:
                         errors.append("Rule #{} is not a valid channel.".format(idx))
                         continue
@@ -562,9 +641,9 @@ class RulesEditor(QtWidgets.QDialog):
                     errors.append("Rule #{} expression returned '{}', but expected '{}'."
                                   .format(idx + 1, act_typ.__name__, exp_typ.__name__))
 
-            except:
-                LOG.exception("Error evaluating Rule #{} expression.".format(idx))
-                errors.append("Rule #{} expression is not valid.".format(idx))
+            except Exception as e:
+                LOG.exception("Error evaluating Rule #{} expression. Exception was: {}".format(idx, e))
+                errors.append("Rule #{} expression is not valid.\n{}".format(idx, e))
 
         if len(errors) > 0:
             error_msg = os.linesep.join(errors)
@@ -577,7 +656,7 @@ class RulesEditor(QtWidgets.QDialog):
         """Save the new rules at the widget `rules` property."""
         # If the form is being edited, we make sure self.rules has all the
         # latest values from the form before we try to validate.  This fixes
-        # a problem where the last form item change wouldn't get saved unless
+        # a problem where the last form item changed wouldn't get saved unless
         # the user knew to hit 'enter' or leave the field to end editing before
         # hitting save.
         if self.frm_edit.isEnabled():
@@ -587,7 +666,7 @@ class RulesEditor(QtWidgets.QDialog):
         is_valid, message = self.is_data_valid()
         if is_valid:
             data = json.dumps(self.rules)
-            print json.dumps(self.rules, sort_keys=True, indent=4)
+            print(json.dumps(self.rules, sort_keys=True, indent=4))
             formWindow = QtDesigner.QDesignerFormWindowInterface.findFormWindow(self.widget)
             if formWindow:
                 formWindow.cursor().setProperty("rules", data)
