@@ -5,7 +5,7 @@ if (sys.version_info > (3, 0)):
           "using a python version greater than 2.7, this is not supported. \n"
           "Make sure that you have the correct version of the libpyqt5.so \n"
           "file in /usr/lib/x86_64-linux-gnu/qt5/plugins/designer/, see the \n"
-          "QtDesigner Plugins section of the QtPyVCP README for more info.")
+          "Installation Trouble Shooting section of the QtPyVCP Documents for more info.")
     sys.exit()
 
 import qtpy
@@ -30,11 +30,14 @@ os.environ['VCP_CONFIG_FILES'] = os.getenv('VCP_CONFIG_FILES', '') + \
 from qtpyvcp.utilities.config_loader import load_config_files_from_env
 CONFIG.update(load_config_files_from_env())
 
-from qtpyvcp.plugins import loadDataPlugins
-loadDataPlugins(CONFIG['data_plugins'])
+from qtpyvcp.app.launcher import loadPlugins
+loadPlugins(CONFIG['data_plugins'])
 
 from qtpyvcp.widgets.form_widgets.designer_plugins import *
 from qtpyvcp.widgets.button_widgets.designer_plugins import *
 from qtpyvcp.widgets.display_widgets.designer_plugins import *
 from qtpyvcp.widgets.input_widgets.designer_plugins import *
 from qtpyvcp.widgets.hal_widgets.designer_plugins import *
+from qtpyvcp.widgets.containers.designer_plugins import *
+
+from qtpyvcp.widgets.external_widgets import *
